## Details of the plugins
#
# For now, everything defaults to being enabled - it's up to the user to specify
#    -DPLUGIN_FOO_ENABLED=FALSE
# to disable a given plugin.
#
# Defining a new plugin:
#    fast_downward_plugin(
#        NAME <NAME>
#        [ DISPLAY_NAME <DISPLAY_NAME> ]
#        [ HELP <HELP> ]
#        SOURCES
#            <FILE_1> [ <FILE_2> ... ]
#        [ DEPENDS <PLUGIN_NAME_1> [ <PLUGIN_NAME_2> ... ] ]
#        [ DEPENDENCY_ONLY ]
#        [ CORE_PLUGIN ]
#    )
#
# <DISPLAY_NAME> defaults to lower case <NAME> and is used to group
#                files in IDEs and for messages.
# <HELP> defaults to <DISPLAY_NAME> and is used to describe the cmake option.
# DEPENDS lists plugins that will be automatically enabled if this plugin
# is enabled. If the dependency was not enabled before, this will be logged.
# DEPENDENCY_ONLY disables the plugin unless it is needed as a dependency and
#     hides the option to enable the plugin in cmake GUIs like ccmake.
# CORE_PLUGIN enables the plugin and hides the option to disable it in
#     cmake GUIs like ccmake.

option(
    DISABLE_PLUGINS_BY_DEFAULT
    "If set to YES only plugins that are specifically enabled will be compiled"
    NO)
# This option should not show up in cmake GUIs like ccmake where all
# plugins are enabled or disabled manually.
mark_as_advanced(DISABLE_PLUGINS_BY_DEFAULT)

set(PLANNER_SOURCES
    main.cpp
    )

# See http://www.fast-downward.org/ForDevelopers/AddingSourceFiles
# for general information on adding source files and CMake plugins.
#
# If you're adding a file to the codebase which *isn't* a plugin, add
# it to the following list. We assume that every *.cc file has a
# corresponding *.h file and add headers to the project automatically.
# For plugin files, see below.

set(CORE_SOURCES
    types.cpp
    cli/macros.cpp
    utils/timer.cpp
    utils/rng.cpp
    storage/array_compressor.cpp
    storage/compressed_array_hash_set.cpp
    storage/conjunctive_condition_evaluator.cpp
    storage/hash.cpp
    storage/int_hash_set.cpp
    storage/segmented_vector.cpp
    storage/state_map.cpp
    sas_task/fact_pair.cpp
    sas_task/variable.cpp
    sas_task/action.cpp
    sas_task/sas_parser_utils.cpp
    sas_task/sas_task.cpp
    sas_task/probabilistic_action.cpp
    sas_task/probabilistic_sas_task.cpp
    models/utils.cpp
    models/transition_cache.cpp
    models/sas/probabilistic_transition_model.cpp
    models/sas/expcost_reward_model.cpp
    models/sas/minprob_reward_model.cpp
    models/sas/maxprob_reward_model.cpp
    heuristics/evaluation_result.cpp
    heuristics/heuristic.cpp
    heuristics/dead_end_heuristic.cpp
    heuristics/classic/cp_heuristic.cpp
    heuristics/classic/hmax_dead_end_heuristic.cpp
    engines/engine.cpp
    engines/fret/fret_model_wrapper.cpp
    engines/fret/fret_pi.cpp
    engines/heuristic_depth_first_search.cpp
    )

fast_downward_add_headers_to_sources_list(CORE_SOURCES)
source_group(core FILES planner.cc ${CORE_SOURCES})
list(APPEND PLANNER_SOURCES ${CORE_SOURCES})

# fast_downward_plugin(
#     NAME OCCUPATION_MEASURE_HEURISTICS
#     SOURCES
#     probabilistic/heuristics/occupation_measure/operator_counting_heuristic.cc
#     DEPENDS LP_SOLVER)

fast_downward_add_plugin_sources(PLANNER_SOURCES)

# The order in PLANNER_SOURCES influences the order in which object
# files are given to the linker, which can have a significant influence
# on performance (see issue67). The general recommendation seems to be
# to list files that define functions after files that use them.
# We approximate this by reversing the list, which will put the plugins
# first, followed by the core files, followed by the main file.
# This is certainly not optimal, but works well enough in practice.
list(REVERSE PLANNER_SOURCES)
