#ifndef CLI_MACROS_H
#define CLI_MACROS_H

#include <cstdlib>
#include <iostream>

#include "../utils/timer.h"

namespace fd {
namespace cli {

enum class ExitCode {
    CLI_PARSE_ERROR      = 1,
    INTERNAL_ERROR       = 2,
    INPUT_ERROR          = 3,
    UNSUPPORTED_FEATURE  = 4,
};

const char* get_exit_message(ExitCode code);

}
}

#define FD_LOG_MESSAGE(msg) {\
    std::cout << msg << std::endl;\
    }

#define FD_LOG_ERROR(msg) {\
    std::cerr << msg << std::endl;\
    }

#define FD_LOG_MESSAGE_T(msg) {\
    std::cout << msg << " [t=" << utils::g_timer << "]" << std::endl;\
    }

#define FD_EXIT(code) {\
    if (fd::cli::get_exit_message(code)) {\
        FD_LOG_MESSAGE(fd::cli::get_exit_message(code));\
    }\
    exit(static_cast<int>(code));\
    }

#define FD_LOG_ERROR_AND_EXIT(code, msg) {\
    FD_LOG_ERROR(msg);\
    FD_EXIT(code);\
    }

#define FD_LOG_EXCEPTION_AND_EXIT(code, msg) {\
    FD_LOG_ERROR(msg);\
    FD_EXIT(code);\
    }

#ifndef NDEBUG

#define FD_LOG_DEBUG_MESSAGE(msg) {\
    std::cout << msg << std::endl;\
    }

#define FD_LOG_DEBUG_MESSAGE_T(msg) {\
    std::cout << msg << " [t=" << utils::g_timer << "]" << std::endl;\
    }
#else

#define FD_LOG_DEBUG_MESSAGE(msg)

#define FD_LOG_DEBUG_MESSAGE_T(msg)

#endif

#endif
