#ifndef DEAD_END_HEURISTIC_H
#define DEAD_END_HEURISTIC_H

#include "heuristic.h"
#include "evaluation_result.h"
#include "classic/cp_heuristic.h"
#include "../types.h"
#include "../storage/compressed_array_hash_set.h"
#include "../storage/array_compressor.h"

#include <memory>
#include <vector>

namespace fd {
namespace heuristics {

template<typename Reward>
class DeadEndHeuristic : public Heuristic<Reward> {
public:
    DeadEndHeuristic(
            std::shared_ptr<classic::ClassicalPlanningHeuristic> cp_heuristic,
            std::shared_ptr<Reward> reward,
            const storage::CompressedArrayHashSet* state_cache);
    virtual EvaluationResult evaluate(const State& state) override;
private:
    std::shared_ptr<classic::ClassicalPlanningHeuristic> m_cp_heuristic;
};

template<typename Reward>
DeadEndHeuristic<Reward>::DeadEndHeuristic(
            std::shared_ptr<classic::ClassicalPlanningHeuristic> cp_heuristic,
            std::shared_ptr<Reward> reward,
            const storage::CompressedArrayHashSet* state_cache)
    : Heuristic<Reward>(reward, state_cache)
    , m_cp_heuristic(cp_heuristic)
{}

template<typename Reward>
EvaluationResult
DeadEndHeuristic<Reward>::evaluate(const State& state)
{
    EvaluationResult res = m_cp_heuristic->evaluate(
            this->m_state_cache->get_compressed_array(state));
    if (res.dead) {
        return EvaluationResult(true, this->m_reward->get_min_reward());
    }
    return EvaluationResult(false, this->m_reward->get_max_reward());
}

}
}

#endif
