#ifndef SAS_MAXPROB_REWARD_MODEL_H
#define SAS_MAXPROB_REWARD_MODEL_H

#include "../../types.h"
#include <memory>

namespace fd {

namespace sas_task {
class ProbabilisticSASTask;
}

namespace storage {
class CompressedArrayHashSet;
}

namespace models {
namespace sas_model {

class MaxProbRewardModel {
public:
    MaxProbRewardModel(
            const std::shared_ptr<sas_task::ProbabilisticSASTask> task,
            const storage::CompressedArrayHashSet& state_cache);

    ValueType get_immediate_reward(const State& s);
    ValueType get_transition_reward(const State& s, const Action& a, const State& t);
    bool      is_terminal(const State& s);
    ValueType transform_reward(const ValueType& v);
    ValueType get_min_reward() const;
    ValueType get_max_reward() const;
private:
    const std::shared_ptr<sas_task::ProbabilisticSASTask> m_task;
    const storage::CompressedArrayHashSet& m_state_cache;
};

}
}
}

#endif
