#ifndef SAS_PROBABILISTIC_TRANSITION_MODEL_H
#define SAS_PROBABILISTIC_TRANSITION_MODEL_H

#include "../../types.h"

#include <memory>
#include <vector>

namespace fd {

namespace sas_task {
class ProbabilisticSASTask;
}

namespace storage {
template<typename X>
class ConjunctiveConditionEvaluator;
class CompressedArrayHashSet;
}

namespace models {
namespace sas_model {

class ProbabilisticTransitionModel {
public:
    ProbabilisticTransitionModel(
            const std::shared_ptr<sas_task::ProbabilisticSASTask> task);
    ~ProbabilisticTransitionModel();

    State get_initial_state();

    void get_applicable_actions(std::vector<Action>& aops, const State& state);

    void get_successors(
            std::vector<std::pair<ValueType, State> >& successors,
            const State& state,
            const Action& action);

    void print_statistics() const;

    const std::shared_ptr<sas_task::ProbabilisticSASTask> get_underlying_task() const;
    const storage::CompressedArrayHashSet& get_state_cache() const;

    void fill_probabilities(
            std::vector<std::pair<ValueType, State> >& successors,
            unsigned start,
            const State& state,
            const Action& action) const;
    unsigned get_num_outcomes(const State& state, const Action& action) const;
private:
    const std::shared_ptr<sas_task::ProbabilisticSASTask> m_task;
    storage::ConjunctiveConditionEvaluator<Action>*  m_aops_generator;
    storage::CompressedArrayHashSet* m_state_cache;

    unsigned m_stat_num_states_generated;
    unsigned m_stat_num_expansions;
};

}
}
}

#endif
