#include "action.h"
#include "sas_parser_utils.h"

#include <cassert>

using namespace std;

namespace fd {
namespace sas_task {

Effect::Effect(
    int var, int value, vector<FactPair>&& conditions)
    : fact(var, value), conditions(move(conditions))
{
}


void
Action::read_pre_post(istream& in)
{
    vector<FactPair> conditions = read_facts(in);
    int var, value_pre, value_post;
    in >> var >> value_pre >> value_post;
    if (value_pre != -1) {
        preconditions.emplace_back(var, value_pre);
    }
    effects.emplace_back(var, value_post, move(conditions));
}

Action::Action(istream& in, bool is_an_axiom,
                                   bool use_metric, unsigned id)
    : is_an_axiom(is_an_axiom), id(id)
{
    if (!is_an_axiom) {
        check_magic(in, "begin_operator");
        in >> ws;
        getline(in, name);
        preconditions = read_facts(in);
        int count;
        in >> count;
        effects.reserve(count);
        for (int i = 0; i < count; ++i) {
            read_pre_post(in);
        }

        int op_cost;
        in >> op_cost;
        cost = use_metric ? op_cost : 1;
        check_magic(in, "end_operator");
    } else {
        name = "<axiom>";
        cost = 0;
        check_magic(in, "begin_rule");
        read_pre_post(in);
        check_magic(in, "end_rule");
    }
    assert(cost >= 0);
}

}
}
