#ifndef SAS_ACTION_H
#define SAS_ACTION_H

#include "fact_pair.h"

#include <vector>
#include <string>
#include <iostream>

namespace fd {
namespace sas_task {

class Effect
{
public:
    Effect() : fact(-1, -1) {}
    Effect(const Effect& e) = default;
    Effect(const FactPair& fact,
                   std::vector<FactPair>&& conds)
        : fact(fact)
        , conditions(std::move(conds)) {}
    Effect(int var, int value, std::vector<FactPair>&& conditions);
    ~Effect() = default;

    const FactPair& get_effect() const { return fact; }
    const std::vector<FactPair>& get_condition() const { return conditions; }
    bool is_conditional_effect() const { return !conditions.empty(); }
private:
    FactPair fact;
    std::vector<FactPair> conditions;
};

class Action
{
public:
    Action(std::istream& in, bool is_an_axiom, bool use_metric, unsigned id);
    Action(const Action& a) = default;
    ~Action() = default;
    const std::vector<FactPair>& get_preconditions() const { return preconditions; }
    const std::vector<Effect>& get_effects() const { return effects; }
    int get_cost() const { return cost; }
    const std::string& get_name() const { return name; }
    bool is_axiom() const { return is_an_axiom; }
    unsigned get_id() const { return id; }
private:
    void read_pre_post(std::istream& in);
    std::vector<FactPair> preconditions;
    std::vector<Effect> effects;
    int cost;
    std::string name;
    bool is_an_axiom;
    unsigned id;
};

}
}

#endif
