#ifndef PROBABILISTIC_SAS_ACTION_H
#define PROBABILISTIC_SAS_ACTION_H

#include "fact_pair.h"
#include "action.h"
#include "../types.h"

#include <vector>
#include <string>

namespace fd {
namespace sas_task {


class ProbabilisticOutcome {
public:
    ProbabilisticOutcome()
        : probability(0) {}
    ProbabilisticOutcome(
        ProbabilisticOutcome&& outcome)
        : effects(std::move(outcome.effects))
        , probability(std::move(outcome.probability))
    {}
    ProbabilisticOutcome(
        std::vector<Effect>&& effects,
        ValueType probability)
        : effects(std::move(effects))
        , probability(probability)
    {}

    std::vector<Effect>& get_effects() { return effects; }
    void set_probability(ValueType p) { probability = p;  }

    const std::vector<Effect>& get_effects() const { return effects; }
    ValueType get_probability() const { return probability; }

    ProbabilisticOutcome canonical() const;
    void dump() const;
private:
    std::vector<Effect> effects;
    ValueType probability;
};



struct ProbabilisticAction
{
public:
    ProbabilisticAction(const std::string& name, unsigned id)
        : name(name), id(id) {}

    std::vector<FactPair>& get_preconditions() { return pre; }
    std::vector<ProbabilisticOutcome >& get_outcomes() { return outcomes; }

    const std::vector<FactPair>& get_preconditions() const { return pre; }
    const std::vector<ProbabilisticOutcome >& get_outcomes() const { return outcomes; }
    const std::string& get_name() const { return name; }
    unsigned get_id() const { return id; }

    void dump() const;

    void normalize();
private:
    std::vector<FactPair> pre;
    std::vector<ProbabilisticOutcome> outcomes;
    std::string name;
    unsigned id;
};



}
}

#endif
