#include "sas_parser_utils.h"
#include "fact_pair.h"
#include "variable.h"
#include "action.h"
#include "../cli/macros.h"

using namespace std;

namespace fd {
namespace sas_task {

int PRE_FILE_VERSION = 3;

void
check_fact(const FactPair& fact, const vector<Variable>& variables)
{
    if (((unsigned) fact.var) >= variables.size()) {
        FD_LOG_ERROR_AND_EXIT(cli::ExitCode::INPUT_ERROR, "Invalid variable id: " << fact.var);
    }
    if (((unsigned) fact.value) >= variables[fact.var].domain_size) {
        FD_LOG_ERROR_AND_EXIT(cli::ExitCode::INPUT_ERROR, "Invalid value for variable " << fact.var << ": " << fact.value);
    }
}

void
check_facts(const vector<FactPair>& facts,
            const vector<Variable>& variables)
{
    for (FactPair fact : facts) {
        check_fact(fact, variables);
    }
}

void
check_facts(const Action& action,
            const vector<Variable>& variables)
{
    check_facts(action.get_preconditions(), variables);
    for (const Effect& eff : action.get_effects()) {
        check_fact(eff.get_effect(), variables);
        check_facts(eff.get_condition(), variables);
    }
}

void
check_magic(istream& in, const string& magic)
{
    string word;
    in >> word;
    if (word != magic) {
        FD_LOG_ERROR("Failed to match magic word '" << magic << "'." << endl
             << "Got '" << word << "'.");
        if (magic == "begin_version") {
            FD_LOG_ERROR_AND_EXIT(cli::ExitCode::INPUT_ERROR, "Possible cause: you are running the planner "
                 << "on a translator output file from " << endl
                 << "an older version.");
        }
    }
}

vector<FactPair>
read_facts(istream& in)
{
    int count;
    in >> count;
    vector<FactPair> conditions;
    conditions.reserve(count);
    for (int i = 0; i < count; ++i) {
        FactPair condition = FactPair::no_fact;
        in >> condition.var >> condition.value;
        conditions.push_back(condition);
    }
    return conditions;
}


}
}
