#ifndef DETERMINISTIC_SAS_TASK_H
#define DETERMINISTIC_SAS_TASK_H

#include "fact_pair.h"
#include "variable.h"
#include "action.h"

#include <string>
#include <vector>
#include <set>
#include <iostream>
#include <memory>

namespace fd {
namespace sas_task {

class SASTask
{
public:
    explicit SASTask(std::istream& in);

    unsigned get_num_variables() const;
    std::string get_variable_name(int var) const;
    unsigned get_variable_domain_size(int var) const;
    int get_variable_axiom_layer(int var) const;
    int get_variable_default_axiom_value(int var) const;
    std::string get_fact_name(const FactPair& fact) const;
    bool are_facts_mutex(
        const FactPair& fact1, const FactPair& fact2) const;

    const std::vector<int>& get_initial_state_values() const;
    unsigned get_num_goals() const;
    const FactPair& get_goal_fact(int x) const;

    unsigned get_num_actions() const;
    unsigned get_num_axioms() const;
    const Action& get_action(int a) const;
    const Action& get_axiom(int a) const;

    bool has_axioms() const;
    bool has_conditional_effects() const;

    const std::set<FactPair>& get_mutexes_for_fact(const FactPair& f) const;
private:
    std::vector<Variable> variables;
    std::vector<std::vector<std::set<FactPair>>> mutexes;
    std::vector<Action> actions;
    std::vector<Action> axioms;
    std::vector<int> initial_state_values;
    std::vector<FactPair> goals;
    bool has_ce;
};

extern std::shared_ptr<SASTask> read_sas_task(std::istream& in);

}
}

#endif
