#ifndef COMPRESSED_ARRAY_HASH_SET_H
#define COMPRESSED_ARRAY_HASH_SET_H

#include "array_compressor.h"
#include "segmented_vector.h"

#include "int_hash_set.h"
//#include <unordered_set>

namespace fd {
namespace storage {

class CompressedArrayHashSet {
public:
    using Id = int_hash_set::KeyType;
    
    CompressedArrayHashSet(const std::vector<int>& bounds);
    ~CompressedArrayHashSet();

    template<typename ValueArray>
    Id insert_get_id(const ValueArray& values);

    Id insert_compressed_get_id(const compression::Bin* compressed_values);

    template<typename ValueArray>
    void get_uncompressed_array(ValueArray& values, const Id& id) const;
    compression::CompressedArrayConstReference get_compressed_array(const Id& id) const;

    const compression::ArrayCompressor& get_compressor() const;

    unsigned size() const;
    unsigned get_num_bytes_per_array() const;

private:
    struct CompressedArrayHasher {
        CompressedArrayHasher(
                const SegmentedArrayVector<compression::Bin>& data,
                unsigned num_bins);

        int_hash_set::HashType
        // std::size_t
            operator()(const Id& x) const;

    private:
        const SegmentedArrayVector<compression::Bin>& data;
        unsigned num_bins;
    };

    struct CompressedArrayEqual {
        CompressedArrayEqual(
                const SegmentedArrayVector<compression::Bin>& data,
                unsigned num_bins);

        bool operator()(const Id& x, const Id& y) const;

    private:
        const SegmentedArrayVector<compression::Bin>& data;
        unsigned num_bins;
    };

    Id insert_id_or_pop_back();

    compression::ArrayCompressor m_compressor;
    SegmentedArrayVector<compression::Bin> m_compressed_arrays;
    int_hash_set::IntHashSet<CompressedArrayHasher, CompressedArrayEqual> m_id_set;
    // std::unordered_set<Id, CompressedArrayHasher, CompressedArrayEqual> m_id_set;
};

template<typename ValueArray>
CompressedArrayHashSet::Id
CompressedArrayHashSet::insert_get_id(const ValueArray& values)
{
    compression::Bin* bin = new compression::Bin[m_compressor.get_num_bins()];
    m_compressor.pack(bin, values);
    m_compressed_arrays.push_back(bin);
    delete[] bin;
    return insert_id_or_pop_back();
}

template<typename ValueArray>
void
CompressedArrayHashSet::get_uncompressed_array(ValueArray& values, const Id& id) const
{
    const compression::Bin* compressed = m_compressed_arrays[id];
    m_compressor.unpack(values, compressed);
}

}
}
#endif
